# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .availability_times import AvailabilityTimes
    from .aws_qasm_3_job_input import AwsQasm3JobInput
    from .aws_qasm_3_job_input_params import AwsQasm3JobInputParams
    from .azure_ionq_job_input import AzureIonqJobInput
    from .azure_ionq_job_input_circuit_item import AzureIonqJobInputCircuitItem
    from .azure_ionq_job_input_circuit_item_type import AzureIonqJobInputCircuitItemType
    from .azure_ionq_job_input_params import AzureIonqJobInputParams
    from .backend import Backend
    from .backend_access_type import BackendAccessType
    from .backend_hardware_provider import BackendHardwareProvider
    from .backend_provider import BackendProvider
    from .backend_state_info import BackendStateInfo
    from .backend_state_info_status import BackendStateInfoStatus
    from .backend_status import BackendStatus
    from .backend_technology import BackendTechnology
    from .backend_type import BackendType
    from .calibration_response import CalibrationResponse
    from .configuration import Configuration
    from .configuration_output_format import ConfigurationOutputFormat
    from .configuration_supported_input_formats_item import ConfigurationSupportedInputFormatsItem
    from .connectivity import Connectivity
    from .cost import Cost
    from .cost_granularity import CostGranularity
    from .documentation import Documentation
    from .documentation_sdks_item import DocumentationSdksItem
    from .gate import Gate
    from .job import Job
    from .job_input_format import JobInputFormat
    from .job_provider import JobProvider
    from .job_sdk_provider import JobSdkProvider
    from .job_status import JobStatus
    from .job_status_response import JobStatusResponse
    from .job_status_response_status import JobStatusResponseStatus
    from .json_node import JsonNode
    from .oauth_scope import OauthScope
    from .page_response_job import PageResponseJob
    from .photonic_properties import PhotonicProperties
    from .qubit import Qubit
    from .service_execution_id import ServiceExecutionId
    from .shots_range import ShotsRange
_dynamic_imports: typing.Dict[str, str] = {
    "AvailabilityTimes": ".availability_times",
    "AwsQasm3JobInput": ".aws_qasm_3_job_input",
    "AwsQasm3JobInputParams": ".aws_qasm_3_job_input_params",
    "AzureIonqJobInput": ".azure_ionq_job_input",
    "AzureIonqJobInputCircuitItem": ".azure_ionq_job_input_circuit_item",
    "AzureIonqJobInputCircuitItemType": ".azure_ionq_job_input_circuit_item_type",
    "AzureIonqJobInputParams": ".azure_ionq_job_input_params",
    "Backend": ".backend",
    "BackendAccessType": ".backend_access_type",
    "BackendHardwareProvider": ".backend_hardware_provider",
    "BackendProvider": ".backend_provider",
    "BackendStateInfo": ".backend_state_info",
    "BackendStateInfoStatus": ".backend_state_info_status",
    "BackendStatus": ".backend_status",
    "BackendTechnology": ".backend_technology",
    "BackendType": ".backend_type",
    "CalibrationResponse": ".calibration_response",
    "Configuration": ".configuration",
    "ConfigurationOutputFormat": ".configuration_output_format",
    "ConfigurationSupportedInputFormatsItem": ".configuration_supported_input_formats_item",
    "Connectivity": ".connectivity",
    "Cost": ".cost",
    "CostGranularity": ".cost_granularity",
    "Documentation": ".documentation",
    "DocumentationSdksItem": ".documentation_sdks_item",
    "Gate": ".gate",
    "Job": ".job",
    "JobInputFormat": ".job_input_format",
    "JobProvider": ".job_provider",
    "JobSdkProvider": ".job_sdk_provider",
    "JobStatus": ".job_status",
    "JobStatusResponse": ".job_status_response",
    "JobStatusResponseStatus": ".job_status_response_status",
    "JsonNode": ".json_node",
    "OauthScope": ".oauth_scope",
    "PageResponseJob": ".page_response_job",
    "PhotonicProperties": ".photonic_properties",
    "Qubit": ".qubit",
    "ServiceExecutionId": ".service_execution_id",
    "ShotsRange": ".shots_range",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AvailabilityTimes",
    "AwsQasm3JobInput",
    "AwsQasm3JobInputParams",
    "AzureIonqJobInput",
    "AzureIonqJobInputCircuitItem",
    "AzureIonqJobInputCircuitItemType",
    "AzureIonqJobInputParams",
    "Backend",
    "BackendAccessType",
    "BackendHardwareProvider",
    "BackendProvider",
    "BackendStateInfo",
    "BackendStateInfoStatus",
    "BackendStatus",
    "BackendTechnology",
    "BackendType",
    "CalibrationResponse",
    "Configuration",
    "ConfigurationOutputFormat",
    "ConfigurationSupportedInputFormatsItem",
    "Connectivity",
    "Cost",
    "CostGranularity",
    "Documentation",
    "DocumentationSdksItem",
    "Gate",
    "Job",
    "JobInputFormat",
    "JobProvider",
    "JobSdkProvider",
    "JobStatus",
    "JobStatusResponse",
    "JobStatusResponseStatus",
    "JsonNode",
    "OauthScope",
    "PageResponseJob",
    "PhotonicProperties",
    "Qubit",
    "ServiceExecutionId",
    "ShotsRange",
]



# This file was auto-generated by Fern from our API Definition.

import typing

ConfigurationSupportedInputFormatsItem = typing.Union[
    typing.Literal[
        "OPEN_QASM_V1",
        "OPEN_QASM_V2",
        "OPEN_QASM_V3",
        "QIR_V1",
        "BRAKET_OPEN_QASM_V3",
        "BRAKET_AHS_PROGRAM",
        "IONQ_CIRCUIT_V1",
        "QISKIT",
        "QOQO",
        "PERCEVAL",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.calibration_response import CalibrationResponse
from ..types.job import Job
from ..types.job_status_response import JobStatusResponse
from ..types.page_response_job import PageResponseJob
from ..types.service_execution_id import ServiceExecutionId
from .raw_client import AsyncRawJobsClient, RawJobsClient
from .types.create_job_request_input import CreateJobRequestInput
from .types.create_job_request_input_format import CreateJobRequestInputFormat
from .types.create_job_request_input_params import CreateJobRequestInputParams

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class JobsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawJobsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawJobsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawJobsClient
        """
        return self._raw_client

    def search_jobs(
        self,
        *,
        page: typing.Optional[int] = None,
        size: typing.Optional[int] = None,
        sort: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        service_execution_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PageResponseJob:
        """
        Get quantum jobs with pagination.

        Parameters
        ----------
        page : typing.Optional[int]
            Zero-based page index (0..N)

        size : typing.Optional[int]
            The size of the page to be returned

        sort : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Sorting criteria in the format: property,(asc|desc). Default sort order is ascending. Multiple sort criteria are supported.

        service_execution_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PageResponseJob
            The page of quantum jobs matching the query.

        Examples
        --------
        from planqk import GeneratedPlanqkQuantumClient

        client = GeneratedPlanqkQuantumClient(
            organizationid="YOUR_ORGANIZATIONID",
            api_key="YOUR_API_KEY",
        )
        client.jobs.search_jobs(
            service_execution_id="X-Service-Execution-Id",
            page=1,
            size=1,
        )
        """
        _response = self._raw_client.search_jobs(
            page=page, size=size, sort=sort, service_execution_id=service_execution_id, request_options=request_options
        )
        return _response.data

    def create_job(
        self,
        *,
        backend_id: str,
        shots: int,
        input: CreateJobRequestInput,
        name: typing.Optional[str] = OMIT,
        input_format: typing.Optional[CreateJobRequestInputFormat] = OMIT,
        input_params: typing.Optional[CreateJobRequestInputParams] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Job:
        """
        Create a new quantum job.

        Parameters
        ----------
        backend_id : str
            Identifier of the backend the job should be executed on

        shots : int
            Number of shots to execute

        input : CreateJobRequestInput
            Backend-specific job input. The format depends on the backend the job is executed on.

        name : typing.Optional[str]
            Optional name for the job

        input_format : typing.Optional[CreateJobRequestInputFormat]
            The input format of the input (e.g. QASM string). The input format must match the backend's capabilities. If not specified, the default format for the backend is assumed.

        input_params : typing.Optional[CreateJobRequestInputParams]
            Backend-specific job input parameters. The supported parameters depend on the backend the job is executed on.

        tags : typing.Optional[typing.Sequence[str]]
            Set of tags to identify or categorize the job

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Job
            The created job.

        Examples
        --------
        from planqk import AzureIonqJobInputCircuitItem, GeneratedPlanqkQuantumClient
        from planqk.jobs import CreateJobRequestInput_AzureIonqSimulator

        client = GeneratedPlanqkQuantumClient(
            organizationid="YOUR_ORGANIZATIONID",
            api_key="YOUR_API_KEY",
        )
        client.jobs.create_job(
            backend_id="aws.sim.sv1",
            shots=1000,
            input=CreateJobRequestInput_AzureIonqSimulator(
                circuit=[
                    AzureIonqJobInputCircuitItem(
                        targets=[0],
                    ),
                    AzureIonqJobInputCircuitItem(
                        targets=[1],
                        controls=[0],
                    ),
                ],
                qubits=2,
            ),
        )
        """
        _response = self._raw_client.create_job(
            backend_id=backend_id,
            shots=shots,
            input=input,
            name=name,
            input_format=input_format,
            input_params=input_params,
            tags=tags,
            request_options=request_options,
        )
        return _response.data

    def get_job(
        self,
        id: str,
        *,
        service_execution_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Job:
        """
        Get a quantum job's details.

        Parameters
        ----------
        id : str

        service_execution_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Job
            The job details.

        Examples
        --------
        from planqk import GeneratedPlanqkQuantumClient

        client = GeneratedPlanqkQuantumClient(
            organizationid="YOUR_ORGANIZATIONID",
            api_key="YOUR_API_KEY",
        )
        client.jobs.get_job(
            id="id",
            service_execution_id="X-Service-Execution-Id",
        )
        """
        _response = self._raw_client.get_job(
            id, service_execution_id=service_execution_id, request_options=request_options
        )
        return _response.data

    def cancel_job(
        self,
        id: str,
        *,
        service_execution_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Cancel the execution of a quantum job.

        Parameters
        ----------
        id : str

        service_execution_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from planqk import GeneratedPlanqkQuantumClient

        client = GeneratedPlanqkQuantumClient(
            organizationid="YOUR_ORGANIZATIONID",
            api_key="YOUR_API_KEY",
        )
        client.jobs.cancel_job(
            id="id",
            service_execution_id="X-Service-Execution-Id",
        )
        """
        _response = self._raw_client.cancel_job(
            id, service_execution_id=service_execution_id, request_options=request_options
        )
        return _response.data

    def get_job_status(
        self,
        id: str,
        *,
        service_execution_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> JobStatusResponse:
        """
        Get a quantum job's status.

        Parameters
        ----------
        id : str

        service_execution_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        JobStatusResponse
            The job's status.

        Examples
        --------
        from planqk import GeneratedPlanqkQuantumClient

        client = GeneratedPlanqkQuantumClient(
            organizationid="YOUR_ORGANIZATIONID",
            api_key="YOUR_API_KEY",
        )
        client.jobs.get_job_status(
            id="id",
            service_execution_id="X-Service-Execution-Id",
        )
        """
        _response = self._raw_client.get_job_status(
            id, service_execution_id=service_execution_id, request_options=request_options
        )
        return _response.data

    def get_job_result(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Any]:
        """
        Get a quantum job's results.

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Any]
            The job's result.

        Examples
        --------
        from planqk import GeneratedPlanqkQuantumClient

        client = GeneratedPlanqkQuantumClient(
            organizationid="YOUR_ORGANIZATIONID",
            api_key="YOUR_API_KEY",
        )
        client.jobs.get_job_result(
            id="id",
        )
        """
        _response = self._raw_client.get_job_result(id, request_options=request_options)
        return _response.data

    def get_job_result_stream(
        self,
        id: str,
        *,
        service_execution_id: typing.Optional[ServiceExecutionId] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[bytes]:
        """
        Get the result of a specific job as json file.

        Parameters
        ----------
        id : str

        service_execution_id : typing.Optional[ServiceExecutionId]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[bytes]
            The job result content as json file

        Examples
        --------
        from planqk import GeneratedPlanqkQuantumClient

        client = GeneratedPlanqkQuantumClient(
            organizationid="YOUR_ORGANIZATIONID",
            api_key="YOUR_API_KEY",
        )
        client.jobs.get_job_result_stream(
            id="id",
        )
        """
        with self._raw_client.get_job_result_stream(
            id, service_execution_id=service_execution_id, request_options=request_options
        ) as r:
            yield from r.data

    def get_job_input(
        self,
        id: str,
        *,
        service_execution_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> str:
        """
        Get the input.json of a quantum job.

        Parameters
        ----------
        id : str

        service_execution_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        str
            The job's input.

        Examples
        --------
        from planqk import GeneratedPlanqkQuantumClient

        client = GeneratedPlanqkQuantumClient(
            organizationid="YOUR_ORGANIZATIONID",
            api_key="YOUR_API_KEY",
        )
        client.jobs.get_job_input(
            id="id",
            service_execution_id="X-Service-Execution-Id",
        )
        """
        _response = self._raw_client.get_job_input(
            id, service_execution_id=service_execution_id, request_options=request_options
        )
        return _response.data

    def get_calibration_of_job_backend(
        self,
        id: str,
        *,
        service_execution_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CalibrationResponse:
        """
        Get the calibration of a specific backend.

        Parameters
        ----------
        id : str

        service_execution_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CalibrationResponse
            The job's calibration.

        Examples
        --------
        from planqk import GeneratedPlanqkQuantumClient

        client = GeneratedPlanqkQuantumClient(
            organizationid="YOUR_ORGANIZATIONID",
            api_key="YOUR_API_KEY",
        )
        client.jobs.get_calibration_of_job_backend(
            id="id",
            service_execution_id="X-Service-Execution-Id",
        )
        """
        _response = self._raw_client.get_calibration_of_job_backend(
            id, service_execution_id=service_execution_id, request_options=request_options
        )
        return _response.data


class AsyncJobsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawJobsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawJobsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawJobsClient
        """
        return self._raw_client

    async def search_jobs(
        self,
        *,
        page: typing.Optional[int] = None,
        size: typing.Optional[int] = None,
        sort: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        service_execution_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PageResponseJob:
        """
        Get quantum jobs with pagination.

        Parameters
        ----------
        page : typing.Optional[int]
            Zero-based page index (0..N)

        size : typing.Optional[int]
            The size of the page to be returned

        sort : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Sorting criteria in the format: property,(asc|desc). Default sort order is ascending. Multiple sort criteria are supported.

        service_execution_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PageResponseJob
            The page of quantum jobs matching the query.

        Examples
        --------
        import asyncio

        from planqk import AsyncGeneratedPlanqkQuantumClient

        client = AsyncGeneratedPlanqkQuantumClient(
            organizationid="YOUR_ORGANIZATIONID",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.jobs.search_jobs(
                service_execution_id="X-Service-Execution-Id",
                page=1,
                size=1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.search_jobs(
            page=page, size=size, sort=sort, service_execution_id=service_execution_id, request_options=request_options
        )
        return _response.data

    async def create_job(
        self,
        *,
        backend_id: str,
        shots: int,
        input: CreateJobRequestInput,
        name: typing.Optional[str] = OMIT,
        input_format: typing.Optional[CreateJobRequestInputFormat] = OMIT,
        input_params: typing.Optional[CreateJobRequestInputParams] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Job:
        """
        Create a new quantum job.

        Parameters
        ----------
        backend_id : str
            Identifier of the backend the job should be executed on

        shots : int
            Number of shots to execute

        input : CreateJobRequestInput
            Backend-specific job input. The format depends on the backend the job is executed on.

        name : typing.Optional[str]
            Optional name for the job

        input_format : typing.Optional[CreateJobRequestInputFormat]
            The input format of the input (e.g. QASM string). The input format must match the backend's capabilities. If not specified, the default format for the backend is assumed.

        input_params : typing.Optional[CreateJobRequestInputParams]
            Backend-specific job input parameters. The supported parameters depend on the backend the job is executed on.

        tags : typing.Optional[typing.Sequence[str]]
            Set of tags to identify or categorize the job

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Job
            The created job.

        Examples
        --------
        import asyncio

        from planqk import (
            AsyncGeneratedPlanqkQuantumClient,
            AzureIonqJobInputCircuitItem,
        )
        from planqk.jobs import CreateJobRequestInput_AzureIonqSimulator

        client = AsyncGeneratedPlanqkQuantumClient(
            organizationid="YOUR_ORGANIZATIONID",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.jobs.create_job(
                backend_id="aws.sim.sv1",
                shots=1000,
                input=CreateJobRequestInput_AzureIonqSimulator(
                    circuit=[
                        AzureIonqJobInputCircuitItem(
                            targets=[0],
                        ),
                        AzureIonqJobInputCircuitItem(
                            targets=[1],
                            controls=[0],
                        ),
                    ],
                    qubits=2,
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_job(
            backend_id=backend_id,
            shots=shots,
            input=input,
            name=name,
            input_format=input_format,
            input_params=input_params,
            tags=tags,
            request_options=request_options,
        )
        return _response.data

    async def get_job(
        self,
        id: str,
        *,
        service_execution_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Job:
        """
        Get a quantum job's details.

        Parameters
        ----------
        id : str

        service_execution_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Job
            The job details.

        Examples
        --------
        import asyncio

        from planqk import AsyncGeneratedPlanqkQuantumClient

        client = AsyncGeneratedPlanqkQuantumClient(
            organizationid="YOUR_ORGANIZATIONID",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.jobs.get_job(
                id="id",
                service_execution_id="X-Service-Execution-Id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_job(
            id, service_execution_id=service_execution_id, request_options=request_options
        )
        return _response.data

    async def cancel_job(
        self,
        id: str,
        *,
        service_execution_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Cancel the execution of a quantum job.

        Parameters
        ----------
        id : str

        service_execution_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from planqk import AsyncGeneratedPlanqkQuantumClient

        client = AsyncGeneratedPlanqkQuantumClient(
            organizationid="YOUR_ORGANIZATIONID",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.jobs.cancel_job(
                id="id",
                service_execution_id="X-Service-Execution-Id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.cancel_job(
            id, service_execution_id=service_execution_id, request_options=request_options
        )
        return _response.data

    async def get_job_status(
        self,
        id: str,
        *,
        service_execution_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> JobStatusResponse:
        """
        Get a quantum job's status.

        Parameters
        ----------
        id : str

        service_execution_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        JobStatusResponse
            The job's status.

        Examples
        --------
        import asyncio

        from planqk import AsyncGeneratedPlanqkQuantumClient

        client = AsyncGeneratedPlanqkQuantumClient(
            organizationid="YOUR_ORGANIZATIONID",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.jobs.get_job_status(
                id="id",
                service_execution_id="X-Service-Execution-Id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_job_status(
            id, service_execution_id=service_execution_id, request_options=request_options
        )
        return _response.data

    async def get_job_result(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Any]:
        """
        Get a quantum job's results.

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Any]
            The job's result.

        Examples
        --------
        import asyncio

        from planqk import AsyncGeneratedPlanqkQuantumClient

        client = AsyncGeneratedPlanqkQuantumClient(
            organizationid="YOUR_ORGANIZATIONID",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.jobs.get_job_result(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_job_result(id, request_options=request_options)
        return _response.data

    async def get_job_result_stream(
        self,
        id: str,
        *,
        service_execution_id: typing.Optional[ServiceExecutionId] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[bytes]:
        """
        Get the result of a specific job as json file.

        Parameters
        ----------
        id : str

        service_execution_id : typing.Optional[ServiceExecutionId]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[bytes]
            The job result content as json file

        Examples
        --------
        import asyncio

        from planqk import AsyncGeneratedPlanqkQuantumClient

        client = AsyncGeneratedPlanqkQuantumClient(
            organizationid="YOUR_ORGANIZATIONID",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.jobs.get_job_result_stream(
                id="id",
            )


        asyncio.run(main())
        """
        async with self._raw_client.get_job_result_stream(
            id, service_execution_id=service_execution_id, request_options=request_options
        ) as r:
            async for _chunk in r.data:
                yield _chunk

    async def get_job_input(
        self,
        id: str,
        *,
        service_execution_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> str:
        """
        Get the input.json of a quantum job.

        Parameters
        ----------
        id : str

        service_execution_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        str
            The job's input.

        Examples
        --------
        import asyncio

        from planqk import AsyncGeneratedPlanqkQuantumClient

        client = AsyncGeneratedPlanqkQuantumClient(
            organizationid="YOUR_ORGANIZATIONID",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.jobs.get_job_input(
                id="id",
                service_execution_id="X-Service-Execution-Id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_job_input(
            id, service_execution_id=service_execution_id, request_options=request_options
        )
        return _response.data

    async def get_calibration_of_job_backend(
        self,
        id: str,
        *,
        service_execution_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CalibrationResponse:
        """
        Get the calibration of a specific backend.

        Parameters
        ----------
        id : str

        service_execution_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CalibrationResponse
            The job's calibration.

        Examples
        --------
        import asyncio

        from planqk import AsyncGeneratedPlanqkQuantumClient

        client = AsyncGeneratedPlanqkQuantumClient(
            organizationid="YOUR_ORGANIZATIONID",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.jobs.get_calibration_of_job_backend(
                id="id",
                service_execution_id="X-Service-Execution-Id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_calibration_of_job_backend(
            id, service_execution_id=service_execution_id, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .cost_granularity import CostGranularity


class Cost(UniversalBaseModel):
    """
    Cost item for a backend
    """

    granularity: typing.Optional[CostGranularity] = pydantic.Field(default=None)
    """
    Cost granularity (e.g. shot, job)
    """

    currency: typing.Optional[str] = pydantic.Field(default=None)
    """
    ISO 4217 currency code
    """

    value: typing.Optional[float] = pydantic.Field(default=None)
    """
    Monetary value per granularity unit
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        CreateJobRequestInput,
        CreateJobRequestInputFormat,
        CreateJobRequestInputParams,
        CreateJobRequestInputParams_AwsIonqAria,
        CreateJobRequestInputParams_AwsIonqAria2,
        CreateJobRequestInputParams_AwsIonqForte,
        CreateJobRequestInputParams_AwsIqmGarnet,
        CreateJobRequestInputParams_AwsRigettiAnkaa,
        CreateJobRequestInputParams_AwsSimDm1,
        CreateJobRequestInputParams_AwsSimSv1,
        CreateJobRequestInputParams_AzureIonqSimulator,
        CreateJobRequestInput_AwsIonqAria,
        CreateJobRequestInput_AwsIonqAria2,
        CreateJobRequestInput_AwsIonqForte,
        CreateJobRequestInput_AwsIqmGarnet,
        CreateJobRequestInput_AwsRigettiAnkaa,
        CreateJobRequestInput_AwsSimDm1,
        CreateJobRequestInput_AwsSimSv1,
        CreateJobRequestInput_AzureIonqSimulator,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "CreateJobRequestInput": ".types",
    "CreateJobRequestInputFormat": ".types",
    "CreateJobRequestInputParams": ".types",
    "CreateJobRequestInputParams_AwsIonqAria": ".types",
    "CreateJobRequestInputParams_AwsIonqAria2": ".types",
    "CreateJobRequestInputParams_AwsIonqForte": ".types",
    "CreateJobRequestInputParams_AwsIqmGarnet": ".types",
    "CreateJobRequestInputParams_AwsRigettiAnkaa": ".types",
    "CreateJobRequestInputParams_AwsSimDm1": ".types",
    "CreateJobRequestInputParams_AwsSimSv1": ".types",
    "CreateJobRequestInputParams_AzureIonqSimulator": ".types",
    "CreateJobRequestInput_AwsIonqAria": ".types",
    "CreateJobRequestInput_AwsIonqAria2": ".types",
    "CreateJobRequestInput_AwsIonqForte": ".types",
    "CreateJobRequestInput_AwsIqmGarnet": ".types",
    "CreateJobRequestInput_AwsRigettiAnkaa": ".types",
    "CreateJobRequestInput_AwsSimDm1": ".types",
    "CreateJobRequestInput_AwsSimSv1": ".types",
    "CreateJobRequestInput_AzureIonqSimulator": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "CreateJobRequestInput",
    "CreateJobRequestInputFormat",
    "CreateJobRequestInputParams",
    "CreateJobRequestInputParams_AwsIonqAria",
    "CreateJobRequestInputParams_AwsIonqAria2",
    "CreateJobRequestInputParams_AwsIonqForte",
    "CreateJobRequestInputParams_AwsIqmGarnet",
    "CreateJobRequestInputParams_AwsRigettiAnkaa",
    "CreateJobRequestInputParams_AwsSimDm1",
    "CreateJobRequestInputParams_AwsSimSv1",
    "CreateJobRequestInputParams_AzureIonqSimulator",
    "CreateJobRequestInput_AwsIonqAria",
    "CreateJobRequestInput_AwsIonqAria2",
    "CreateJobRequestInput_AwsIonqForte",
    "CreateJobRequestInput_AwsIqmGarnet",
    "CreateJobRequestInput_AwsRigettiAnkaa",
    "CreateJobRequestInput_AwsSimDm1",
    "CreateJobRequestInput_AwsSimSv1",
    "CreateJobRequestInput_AzureIonqSimulator",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.aws_qasm_3_job_input import AwsQasm3JobInput
from ...types.azure_ionq_job_input_circuit_item import AzureIonqJobInputCircuitItem


class CreateJobRequestInput_AzureIonqSimulator(UniversalBaseModel):
    """
    Backend-specific job input. The format depends on the backend the job is executed on.
    """

    backend_id: typing_extensions.Annotated[
        typing.Literal["azure.ionq.simulator"], FieldMetadata(alias="backendId")
    ] = "azure.ionq.simulator"
    circuit: typing.List[AzureIonqJobInputCircuitItem]
    gateset: typing.Literal["qis"] = "qis"
    qubits: int
    registers: typing.Optional[typing.Dict[str, typing.List[int]]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class CreateJobRequestInput_AwsIqmGarnet(UniversalBaseModel):
    value: AwsQasm3JobInput
    backend_id: typing_extensions.Annotated[typing.Literal["aws.iqm.garnet"], FieldMetadata(alias="backendId")] = (
        "aws.iqm.garnet"
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


class CreateJobRequestInput_AwsSimSv1(UniversalBaseModel):
    value: AwsQasm3JobInput
    backend_id: typing_extensions.Annotated[typing.Literal["aws.sim.sv1"], FieldMetadata(alias="backendId")] = (
        "aws.sim.sv1"
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


class CreateJobRequestInput_AwsIonqAria2(UniversalBaseModel):
    value: AwsQasm3JobInput
    backend_id: typing_extensions.Annotated[typing.Literal["aws.ionq.aria-2"], FieldMetadata(alias="backendId")] = (
        "aws.ionq.aria-2"
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


class CreateJobRequestInput_AwsIonqForte(UniversalBaseModel):
    value: AwsQasm3JobInput
    backend_id: typing_extensions.Annotated[typing.Literal["aws.ionq.forte"], FieldMetadata(alias="backendId")] = (
        "aws.ionq.forte"
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


class CreateJobRequestInput_AwsSimDm1(UniversalBaseModel):
    value: AwsQasm3JobInput
    backend_id: typing_extensions.Annotated[typing.Literal["aws.sim.dm1"], FieldMetadata(alias="backendId")] = (
        "aws.sim.dm1"
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


class CreateJobRequestInput_AwsIonqAria(UniversalBaseModel):
    value: AwsQasm3JobInput
    backend_id: typing_extensions.Annotated[typing.Literal["aws.ionq.aria"], FieldMetadata(alias="backendId")] = (
        "aws.ionq.aria"
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


class CreateJobRequestInput_AwsRigettiAnkaa(UniversalBaseModel):
    value: AwsQasm3JobInput
    backend_id: typing_extensions.Annotated[typing.Literal["aws.rigetti.ankaa"], FieldMetadata(alias="backendId")] = (
        "aws.rigetti.ankaa"
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


CreateJobRequestInput = typing_extensions.Annotated[
    typing.Union[
        CreateJobRequestInput_AzureIonqSimulator,
        CreateJobRequestInput_AwsIqmGarnet,
        CreateJobRequestInput_AwsSimSv1,
        CreateJobRequestInput_AwsIonqAria2,
        CreateJobRequestInput_AwsIonqForte,
        CreateJobRequestInput_AwsSimDm1,
        CreateJobRequestInput_AwsIonqAria,
        CreateJobRequestInput_AwsRigettiAnkaa,
    ],
    pydantic.Field(discriminator="backendId"),
]



# This file was auto-generated by Fern from our API Definition.

import typing

BackendHardwareProvider = typing.Union[
    typing.Literal[
        "IONQ",
        "RIGETTI",
        "OQC",
        "AWS",
        "AZURE",
        "IBM",
        "DWAVE",
        "QRYD",
        "QUERA",
        "IQM",
        "QUDORA",
        "QUANTINUUM",
        "QUANDELA",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .availability_times import AvailabilityTimes
from .backend_access_type import BackendAccessType
from .backend_hardware_provider import BackendHardwareProvider
from .backend_provider import BackendProvider
from .backend_status import BackendStatus
from .backend_technology import BackendTechnology
from .backend_type import BackendType
from .configuration import Configuration
from .cost import Cost
from .documentation import Documentation


class Backend(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier of the backend
    """

    internal_id: typing.Optional[str] = None
    provider: typing.Optional[BackendProvider] = None
    hardware_provider: typing.Optional[BackendHardwareProvider] = None
    name: typing.Optional[str] = None
    display_name: typing.Optional[str] = None
    type: typing.Optional[BackendType] = None
    technology: typing.Optional[BackendTechnology] = None
    status: typing.Optional[BackendStatus] = None
    queue_size: typing.Optional[int] = None
    updated_at: typing.Optional[str] = None
    access_type: typing.Optional[BackendAccessType] = None
    documentation: typing.Optional[Documentation] = None
    configuration: typing.Optional[Configuration] = None
    availability: typing.Optional[typing.List[AvailabilityTimes]] = None
    costs: typing.Optional[typing.List[Cost]] = None
    has_calibration: typing.Optional[bool] = None
    free_of_charge: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        AvailabilityTimes,
        AwsQasm3JobInput,
        AwsQasm3JobInputParams,
        AzureIonqJobInput,
        AzureIonqJobInputCircuitItem,
        AzureIonqJobInputCircuitItemType,
        AzureIonqJobInputParams,
        Backend,
        BackendAccessType,
        BackendHardwareProvider,
        BackendProvider,
        BackendStateInfo,
        BackendStateInfoStatus,
        BackendStatus,
        BackendTechnology,
        BackendType,
        CalibrationResponse,
        Configuration,
        ConfigurationOutputFormat,
        ConfigurationSupportedInputFormatsItem,
        Connectivity,
        Cost,
        CostGranularity,
        Documentation,
        DocumentationSdksItem,
        Gate,
        Job,
        JobInputFormat,
        JobProvider,
        JobSdkProvider,
        JobStatus,
        JobStatusResponse,
        JobStatusResponseStatus,
        JsonNode,
        OauthScope,
        PageResponseJob,
        PhotonicProperties,
        Qubit,
        ServiceExecutionId,
        ShotsRange,
    )
    from .errors import ForbiddenError, InternalServerError, NotFoundError, UnauthorizedError
    from . import backends, jobs
    from .client import AsyncGeneratedPlanqkQuantumClient, GeneratedPlanqkQuantumClient
    from .environment import GeneratedPlanqkQuantumClientEnvironment
    from .jobs import (
        CreateJobRequestInput,
        CreateJobRequestInputFormat,
        CreateJobRequestInputParams,
        CreateJobRequestInputParams_AwsIonqAria,
        CreateJobRequestInputParams_AwsIonqAria2,
        CreateJobRequestInputParams_AwsIonqForte,
        CreateJobRequestInputParams_AwsIqmGarnet,
        CreateJobRequestInputParams_AwsRigettiAnkaa,
        CreateJobRequestInputParams_AwsSimDm1,
        CreateJobRequestInputParams_AwsSimSv1,
        CreateJobRequestInputParams_AzureIonqSimulator,
        CreateJobRequestInput_AwsIonqAria,
        CreateJobRequestInput_AwsIonqAria2,
        CreateJobRequestInput_AwsIonqForte,
        CreateJobRequestInput_AwsIqmGarnet,
        CreateJobRequestInput_AwsRigettiAnkaa,
        CreateJobRequestInput_AwsSimDm1,
        CreateJobRequestInput_AwsSimSv1,
        CreateJobRequestInput_AzureIonqSimulator,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "AsyncGeneratedPlanqkQuantumClient": ".client",
    "AvailabilityTimes": ".types",
    "AwsQasm3JobInput": ".types",
    "AwsQasm3JobInputParams": ".types",
    "AzureIonqJobInput": ".types",
    "AzureIonqJobInputCircuitItem": ".types",
    "AzureIonqJobInputCircuitItemType": ".types",
    "AzureIonqJobInputParams": ".types",
    "Backend": ".types",
    "BackendAccessType": ".types",
    "BackendHardwareProvider": ".types",
    "BackendProvider": ".types",
    "BackendStateInfo": ".types",
    "BackendStateInfoStatus": ".types",
    "BackendStatus": ".types",
    "BackendTechnology": ".types",
    "BackendType": ".types",
    "CalibrationResponse": ".types",
    "Configuration": ".types",
    "ConfigurationOutputFormat": ".types",
    "ConfigurationSupportedInputFormatsItem": ".types",
    "Connectivity": ".types",
    "Cost": ".types",
    "CostGranularity": ".types",
    "CreateJobRequestInput": ".jobs",
    "CreateJobRequestInputFormat": ".jobs",
    "CreateJobRequestInputParams": ".jobs",
    "CreateJobRequestInputParams_AwsIonqAria": ".jobs",
    "CreateJobRequestInputParams_AwsIonqAria2": ".jobs",
    "CreateJobRequestInputParams_AwsIonqForte": ".jobs",
    "CreateJobRequestInputParams_AwsIqmGarnet": ".jobs",
    "CreateJobRequestInputParams_AwsRigettiAnkaa": ".jobs",
    "CreateJobRequestInputParams_AwsSimDm1": ".jobs",
    "CreateJobRequestInputParams_AwsSimSv1": ".jobs",
    "CreateJobRequestInputParams_AzureIonqSimulator": ".jobs",
    "CreateJobRequestInput_AwsIonqAria": ".jobs",
    "CreateJobRequestInput_AwsIonqAria2": ".jobs",
    "CreateJobRequestInput_AwsIonqForte": ".jobs",
    "CreateJobRequestInput_AwsIqmGarnet": ".jobs",
    "CreateJobRequestInput_AwsRigettiAnkaa": ".jobs",
    "CreateJobRequestInput_AwsSimDm1": ".jobs",
    "CreateJobRequestInput_AwsSimSv1": ".jobs",
    "CreateJobRequestInput_AzureIonqSimulator": ".jobs",
    "Documentation": ".types",
    "DocumentationSdksItem": ".types",
    "ForbiddenError": ".errors",
    "Gate": ".types",
    "GeneratedPlanqkQuantumClient": ".client",
    "GeneratedPlanqkQuantumClientEnvironment": ".environment",
    "InternalServerError": ".errors",
    "Job": ".types",
    "JobInputFormat": ".types",
    "JobProvider": ".types",
    "JobSdkProvider": ".types",
    "JobStatus": ".types",
    "JobStatusResponse": ".types",
    "JobStatusResponseStatus": ".types",
    "JsonNode": ".types",
    "NotFoundError": ".errors",
    "OauthScope": ".types",
    "PageResponseJob": ".types",
    "PhotonicProperties": ".types",
    "Qubit": ".types",
    "ServiceExecutionId": ".types",
    "ShotsRange": ".types",
    "UnauthorizedError": ".errors",
    "backends": ".backends",
    "jobs": ".jobs",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AsyncGeneratedPlanqkQuantumClient",
    "AvailabilityTimes",
    "AwsQasm3JobInput",
    "AwsQasm3JobInputParams",
    "AzureIonqJobInput",
    "AzureIonqJobInputCircuitItem",
    "AzureIonqJobInputCircuitItemType",
    "AzureIonqJobInputParams",
    "Backend",
    "BackendAccessType",
    "BackendHardwareProvider",
    "BackendProvider",
    "BackendStateInfo",
    "BackendStateInfoStatus",
    "BackendStatus",
    "BackendTechnology",
    "BackendType",
    "CalibrationResponse",
    "Configuration",
    "ConfigurationOutputFormat",
    "ConfigurationSupportedInputFormatsItem",
    "Connectivity",
    "Cost",
    "CostGranularity",
    "CreateJobRequestInput",
    "CreateJobRequestInputFormat",
    "CreateJobRequestInputParams",
    "CreateJobRequestInputParams_AwsIonqAria",
    "CreateJobRequestInputParams_AwsIonqAria2",
    "CreateJobRequestInputParams_AwsIonqForte",
    "CreateJobRequestInputParams_AwsIqmGarnet",
    "CreateJobRequestInputParams_AwsRigettiAnkaa",
    "CreateJobRequestInputParams_AwsSimDm1",
    "CreateJobRequestInputParams_AwsSimSv1",
    "CreateJobRequestInputParams_AzureIonqSimulator",
    "CreateJobRequestInput_AwsIonqAria",
    "CreateJobRequestInput_AwsIonqAria2",
    "CreateJobRequestInput_AwsIonqForte",
    "CreateJobRequestInput_AwsIqmGarnet",
    "CreateJobRequestInput_AwsRigettiAnkaa",
    "CreateJobRequestInput_AwsSimDm1",
    "CreateJobRequestInput_AwsSimSv1",
    "CreateJobRequestInput_AzureIonqSimulator",
    "Documentation",
    "DocumentationSdksItem",
    "ForbiddenError",
    "Gate",
    "GeneratedPlanqkQuantumClient",
    "GeneratedPlanqkQuantumClientEnvironment",
    "InternalServerError",
    "Job",
    "JobInputFormat",
    "JobProvider",
    "JobSdkProvider",
    "JobStatus",
    "JobStatusResponse",
    "JobStatusResponseStatus",
    "JsonNode",
    "NotFoundError",
    "OauthScope",
    "PageResponseJob",
    "PhotonicProperties",
    "Qubit",
    "ServiceExecutionId",
    "ShotsRange",
    "UnauthorizedError",
    "backends",
    "jobs",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class CreateJobRequestInputParams_AzureIonqSimulator(UniversalBaseModel):
    """
    Backend-specific job input parameters. The supported parameters depend on the backend the job is executed on.
    """

    backend_id: typing_extensions.Annotated[
        typing.Literal["azure.ionq.simulator"], FieldMetadata(alias="backendId")
    ] = "azure.ionq.simulator"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class CreateJobRequestInputParams_AwsIqmGarnet(UniversalBaseModel):
    """
    Backend-specific job input parameters. The supported parameters depend on the backend the job is executed on.
    """

    backend_id: typing_extensions.Annotated[typing.Literal["aws.iqm.garnet"], FieldMetadata(alias="backendId")] = (
        "aws.iqm.garnet"
    )
    disable_qubit_rewiring: typing.Optional[bool] = None
    qubit_count: typing.Optional[float] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class CreateJobRequestInputParams_AwsSimSv1(UniversalBaseModel):
    """
    Backend-specific job input parameters. The supported parameters depend on the backend the job is executed on.
    """

    backend_id: typing_extensions.Annotated[typing.Literal["aws.sim.sv1"], FieldMetadata(alias="backendId")] = (
        "aws.sim.sv1"
    )
    disable_qubit_rewiring: typing.Optional[bool] = None
    qubit_count: typing.Optional[float] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class CreateJobRequestInputParams_AwsIonqAria2(UniversalBaseModel):
    """
    Backend-specific job input parameters. The supported parameters depend on the backend the job is executed on.
    """

    backend_id: typing_extensions.Annotated[typing.Literal["aws.ionq.aria-2"], FieldMetadata(alias="backendId")] = (
        "aws.ionq.aria-2"
    )
    disable_qubit_rewiring: typing.Optional[bool] = None
    qubit_count: typing.Optional[float] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class CreateJobRequestInputParams_AwsIonqForte(UniversalBaseModel):
    """
    Backend-specific job input parameters. The supported parameters depend on the backend the job is executed on.
    """

    backend_id: typing_extensions.Annotated[typing.Literal["aws.ionq.forte"], FieldMetadata(alias="backendId")] = (
        "aws.ionq.forte"
    )
    disable_qubit_rewiring: typing.Optional[bool] = None
    qubit_count: typing.Optional[float] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class CreateJobRequestInputParams_AwsSimDm1(UniversalBaseModel):
    """
    Backend-specific job input parameters. The supported parameters depend on the backend the job is executed on.
    """

    backend_id: typing_extensions.Annotated[typing.Literal["aws.sim.dm1"], FieldMetadata(alias="backendId")] = (
        "aws.sim.dm1"
    )
    disable_qubit_rewiring: typing.Optional[bool] = None
    qubit_count: typing.Optional[float] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class CreateJobRequestInputParams_AwsIonqAria(UniversalBaseModel):
    """
    Backend-specific job input parameters. The supported parameters depend on the backend the job is executed on.
    """

    backend_id: typing_extensions.Annotated[typing.Literal["aws.ionq.aria"], FieldMetadata(alias="backendId")] = (
        "aws.ionq.aria"
    )
    disable_qubit_rewiring: typing.Optional[bool] = None
    qubit_count: typing.Optional[float] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class CreateJobRequestInputParams_AwsRigettiAnkaa(UniversalBaseModel):
    """
    Backend-specific job input parameters. The supported parameters depend on the backend the job is executed on.
    """

    backend_id: typing_extensions.Annotated[typing.Literal["aws.rigetti.ankaa"], FieldMetadata(alias="backendId")] = (
        "aws.rigetti.ankaa"
    )
    disable_qubit_rewiring: typing.Optional[bool] = None
    qubit_count: typing.Optional[float] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


CreateJobRequestInputParams = typing_extensions.Annotated[
    typing.Union[
        CreateJobRequestInputParams_AzureIonqSimulator,
        CreateJobRequestInputParams_AwsIqmGarnet,
        CreateJobRequestInputParams_AwsSimSv1,
        CreateJobRequestInputParams_AwsIonqAria2,
        CreateJobRequestInputParams_AwsIonqForte,
        CreateJobRequestInputParams_AwsSimDm1,
        CreateJobRequestInputParams_AwsIonqAria,
        CreateJobRequestInputParams_AwsRigettiAnkaa,
    ],
    pydantic.Field(discriminator="backendId"),
]



# This file was auto-generated by Fern from our API Definition.

import typing

BackendTechnology = typing.Union[
    typing.Literal["SUPERCONDUCTING", "TRAPPED_ION", "PHOTONIC", "NEUTRAL_ATOMS", "UNKNOWN"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import contextlib
import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.forbidden_error import ForbiddenError
from ..errors.internal_server_error import InternalServerError
from ..errors.not_found_error import NotFoundError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.calibration_response import CalibrationResponse
from ..types.job import Job
from ..types.job_status_response import JobStatusResponse
from ..types.page_response_job import PageResponseJob
from ..types.service_execution_id import ServiceExecutionId
from .types.create_job_request_input import CreateJobRequestInput
from .types.create_job_request_input_format import CreateJobRequestInputFormat
from .types.create_job_request_input_params import CreateJobRequestInputParams

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawJobsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def search_jobs(
        self,
        *,
        page: typing.Optional[int] = None,
        size: typing.Optional[int] = None,
        sort: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        service_execution_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PageResponseJob]:
        """
        Get quantum jobs with pagination.

        Parameters
        ----------
        page : typing.Optional[int]
            Zero-based page index (0..N)

        size : typing.Optional[int]
            The size of the page to be returned

        sort : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Sorting criteria in the format: property,(asc|desc). Default sort order is ascending. Multiple sort criteria are supported.

        service_execution_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PageResponseJob]
            The page of quantum jobs matching the query.
        """
        _response = self._client_wrapper.httpx_client.request(
            "jobs",
            method="GET",
            params={
                "page": page,
                "size": size,
                "sort": sort,
            },
            headers={
                "X-Service-Execution-Id": str(service_execution_id) if service_execution_id is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PageResponseJob,
                    parse_obj_as(
                        type_=PageResponseJob,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_job(
        self,
        *,
        backend_id: str,
        shots: int,
        input: CreateJobRequestInput,
        name: typing.Optional[str] = OMIT,
        input_format: typing.Optional[CreateJobRequestInputFormat] = OMIT,
        input_params: typing.Optional[CreateJobRequestInputParams] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Job]:
        """
        Create a new quantum job.

        Parameters
        ----------
        backend_id : str
            Identifier of the backend the job should be executed on

        shots : int
            Number of shots to execute

        input : CreateJobRequestInput
            Backend-specific job input. The format depends on the backend the job is executed on.

        name : typing.Optional[str]
            Optional name for the job

        input_format : typing.Optional[CreateJobRequestInputFormat]
            The input format of the input (e.g. QASM string). The input format must match the backend's capabilities. If not specified, the default format for the backend is assumed.

        input_params : typing.Optional[CreateJobRequestInputParams]
            Backend-specific job input parameters. The supported parameters depend on the backend the job is executed on.

        tags : typing.Optional[typing.Sequence[str]]
            Set of tags to identify or categorize the job

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Job]
            The created job.
        """
        _response = self._client_wrapper.httpx_client.request(
            "jobs",
            method="POST",
            json={
                "name": name,
                "backend_id": backend_id,
                "shots": shots,
                "input": convert_and_respect_annotation_metadata(
                    object_=input, annotation=CreateJobRequestInput, direction="write"
                ),
                "input_format": input_format,
                "input_params": convert_and_respect_annotation_metadata(
                    object_=input_params, annotation=CreateJobRequestInputParams, direction="write"
                ),
                "tags": tags,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Job,
                    parse_obj_as(
                        type_=Job,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_job(
        self,
        id: str,
        *,
        service_execution_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Job]:
        """
        Get a quantum job's details.

        Parameters
        ----------
        id : str

        service_execution_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Job]
            The job details.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"jobs/{jsonable_encoder(id)}",
            method="GET",
            headers={
                "X-Service-Execution-Id": str(service_execution_id) if service_execution_id is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Job,
                    parse_obj_as(
                        type_=Job,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def cancel_job(
        self,
        id: str,
        *,
        service_execution_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[None]:
        """
        Cancel the execution of a quantum job.

        Parameters
        ----------
        id : str

        service_execution_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"jobs/{jsonable_encoder(id)}",
            method="DELETE",
            headers={
                "X-Service-Execution-Id": str(service_execution_id) if service_execution_id is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_job_status(
        self,
        id: str,
        *,
        service_execution_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[JobStatusResponse]:
        """
        Get a quantum job's status.

        Parameters
        ----------
        id : str

        service_execution_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[JobStatusResponse]
            The job's status.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"jobs/{jsonable_encoder(id)}/status",
            method="GET",
            headers={
                "X-Service-Execution-Id": str(service_execution_id) if service_execution_id is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    JobStatusResponse,
                    parse_obj_as(
                        type_=JobStatusResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_job_result(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.Dict[str, typing.Any]]:
        """
        Get a quantum job's results.

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Dict[str, typing.Any]]
            The job's result.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"jobs/{jsonable_encoder(id)}/result",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Dict[str, typing.Any],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    @contextlib.contextmanager
    def get_job_result_stream(
        self,
        id: str,
        *,
        service_execution_id: typing.Optional[ServiceExecutionId] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[HttpResponse[typing.Iterator[bytes]]]:
        """
        Get the result of a specific job as json file.

        Parameters
        ----------
        id : str

        service_execution_id : typing.Optional[ServiceExecutionId]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[HttpResponse[typing.Iterator[bytes]]]
            The job result content as json file
        """
        with self._client_wrapper.httpx_client.stream(
            f"jobs/{jsonable_encoder(id)}/result-stream",
            method="GET",
            headers={
                "X-Service-Execution-Id": str(service_execution_id) if service_execution_id is not None else None,
            },
            request_options=request_options,
        ) as _response:

            def _stream() -> HttpResponse[typing.Iterator[bytes]]:
                try:
                    if 200 <= _response.status_code < 300:
                        _chunk_size = request_options.get("chunk_size", None) if request_options is not None else None
                        return HttpResponse(
                            response=_response, data=(_chunk for _chunk in _response.iter_bytes(chunk_size=_chunk_size))
                        )
                    _response.read()
                    if _response.status_code == 401:
                        raise UnauthorizedError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                parse_obj_as(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 403:
                        raise ForbiddenError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                parse_obj_as(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 404:
                        raise NotFoundError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                parse_obj_as(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 500:
                        raise InternalServerError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                parse_obj_as(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield _stream()

    def get_job_input(
        self,
        id: str,
        *,
        service_execution_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[str]:
        """
        Get the input.json of a quantum job.

        Parameters
        ----------
        id : str

        service_execution_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[str]
            The job's input.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"jobs/{jsonable_encoder(id)}/input",
            method="GET",
            headers={
                "X-Service-Execution-Id": str(service_execution_id) if service_execution_id is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    str,
                    parse_obj_as(
                        type_=str,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_calibration_of_job_backend(
        self,
        id: str,
        *,
        service_execution_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CalibrationResponse]:
        """
        Get the calibration of a specific backend.

        Parameters
        ----------
        id : str

        service_execution_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CalibrationResponse]
            The job's calibration.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"jobs/{jsonable_encoder(id)}/calibration",
            method="GET",
            headers={
                "X-Service-Execution-Id": str(service_execution_id) if service_execution_id is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CalibrationResponse,
                    parse_obj_as(
                        type_=CalibrationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawJobsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def search_jobs(
        self,
        *,
        page: typing.Optional[int] = None,
        size: typing.Optional[int] = None,
        sort: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        service_execution_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PageResponseJob]:
        """
        Get quantum jobs with pagination.

        Parameters
        ----------
        page : typing.Optional[int]
            Zero-based page index (0..N)

        size : typing.Optional[int]
            The size of the page to be returned

        sort : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Sorting criteria in the format: property,(asc|desc). Default sort order is ascending. Multiple sort criteria are supported.

        service_execution_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PageResponseJob]
            The page of quantum jobs matching the query.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "jobs",
            method="GET",
            params={
                "page": page,
                "size": size,
                "sort": sort,
            },
            headers={
                "X-Service-Execution-Id": str(service_execution_id) if service_execution_id is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PageResponseJob,
                    parse_obj_as(
                        type_=PageResponseJob,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_job(
        self,
        *,
        backend_id: str,
        shots: int,
        input: CreateJobRequestInput,
        name: typing.Optional[str] = OMIT,
        input_format: typing.Optional[CreateJobRequestInputFormat] = OMIT,
        input_params: typing.Optional[CreateJobRequestInputParams] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Job]:
        """
        Create a new quantum job.

        Parameters
        ----------
        backend_id : str
            Identifier of the backend the job should be executed on

        shots : int
            Number of shots to execute

        input : CreateJobRequestInput
            Backend-specific job input. The format depends on the backend the job is executed on.

        name : typing.Optional[str]
            Optional name for the job

        input_format : typing.Optional[CreateJobRequestInputFormat]
            The input format of the input (e.g. QASM string). The input format must match the backend's capabilities. If not specified, the default format for the backend is assumed.

        input_params : typing.Optional[CreateJobRequestInputParams]
            Backend-specific job input parameters. The supported parameters depend on the backend the job is executed on.

        tags : typing.Optional[typing.Sequence[str]]
            Set of tags to identify or categorize the job

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Job]
            The created job.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "jobs",
            method="POST",
            json={
                "name": name,
                "backend_id": backend_id,
                "shots": shots,
                "input": convert_and_respect_annotation_metadata(
                    object_=input, annotation=CreateJobRequestInput, direction="write"
                ),
                "input_format": input_format,
                "input_params": convert_and_respect_annotation_metadata(
                    object_=input_params, annotation=CreateJobRequestInputParams, direction="write"
                ),
                "tags": tags,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Job,
                    parse_obj_as(
                        type_=Job,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_job(
        self,
        id: str,
        *,
        service_execution_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Job]:
        """
        Get a quantum job's details.

        Parameters
        ----------
        id : str

        service_execution_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Job]
            The job details.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"jobs/{jsonable_encoder(id)}",
            method="GET",
            headers={
                "X-Service-Execution-Id": str(service_execution_id) if service_execution_id is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Job,
                    parse_obj_as(
                        type_=Job,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def cancel_job(
        self,
        id: str,
        *,
        service_execution_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[None]:
        """
        Cancel the execution of a quantum job.

        Parameters
        ----------
        id : str

        service_execution_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"jobs/{jsonable_encoder(id)}",
            method="DELETE",
            headers={
                "X-Service-Execution-Id": str(service_execution_id) if service_execution_id is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_job_status(
        self,
        id: str,
        *,
        service_execution_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[JobStatusResponse]:
        """
        Get a quantum job's status.

        Parameters
        ----------
        id : str

        service_execution_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[JobStatusResponse]
            The job's status.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"jobs/{jsonable_encoder(id)}/status",
            method="GET",
            headers={
                "X-Service-Execution-Id": str(service_execution_id) if service_execution_id is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    JobStatusResponse,
                    parse_obj_as(
                        type_=JobStatusResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_job_result(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.Dict[str, typing.Any]]:
        """
        Get a quantum job's results.

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Dict[str, typing.Any]]
            The job's result.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"jobs/{jsonable_encoder(id)}/result",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Dict[str, typing.Any],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    @contextlib.asynccontextmanager
    async def get_job_result_stream(
        self,
        id: str,
        *,
        service_execution_id: typing.Optional[ServiceExecutionId] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[bytes]]]:
        """
        Get the result of a specific job as json file.

        Parameters
        ----------
        id : str

        service_execution_id : typing.Optional[ServiceExecutionId]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[bytes]]]
            The job result content as json file
        """
        async with self._client_wrapper.httpx_client.stream(
            f"jobs/{jsonable_encoder(id)}/result-stream",
            method="GET",
            headers={
                "X-Service-Execution-Id": str(service_execution_id) if service_execution_id is not None else None,
            },
            request_options=request_options,
        ) as _response:

            async def _stream() -> AsyncHttpResponse[typing.AsyncIterator[bytes]]:
                try:
                    if 200 <= _response.status_code < 300:
                        _chunk_size = request_options.get("chunk_size", None) if request_options is not None else None
                        return AsyncHttpResponse(
                            response=_response,
                            data=(_chunk async for _chunk in _response.aiter_bytes(chunk_size=_chunk_size)),
                        )
                    await _response.aread()
                    if _response.status_code == 401:
                        raise UnauthorizedError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                parse_obj_as(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 403:
                        raise ForbiddenError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                parse_obj_as(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 404:
                        raise NotFoundError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                parse_obj_as(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 500:
                        raise InternalServerError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                parse_obj_as(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield await _stream()

    async def get_job_input(
        self,
        id: str,
        *,
        service_execution_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[str]:
        """
        Get the input.json of a quantum job.

        Parameters
        ----------
        id : str

        service_execution_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[str]
            The job's input.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"jobs/{jsonable_encoder(id)}/input",
            method="GET",
            headers={
                "X-Service-Execution-Id": str(service_execution_id) if service_execution_id is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    str,
                    parse_obj_as(
                        type_=str,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_calibration_of_job_backend(
        self,
        id: str,
        *,
        service_execution_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CalibrationResponse]:
        """
        Get the calibration of a specific backend.

        Parameters
        ----------
        id : str

        service_execution_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CalibrationResponse]
            The job's calibration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"jobs/{jsonable_encoder(id)}/calibration",
            method="GET",
            headers={
                "X-Service-Execution-Id": str(service_execution_id) if service_execution_id is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CalibrationResponse,
                    parse_obj_as(
                        type_=CalibrationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .job_input_format import JobInputFormat
from .job_provider import JobProvider
from .job_sdk_provider import JobSdkProvider
from .job_status import JobStatus
from .json_node import JsonNode


class Job(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier of the job within PLANQK
    """

    name: typing.Optional[str] = None
    backend_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="backendId")] = pydantic.Field(
        default=None
    )
    """
    Unique identifier of the backend
    """

    provider: typing.Optional[JobProvider] = None
    provider_job_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="providerJobId")] = (
        pydantic.Field(default=None)
    )
    """
    Unique identifier of the job assigned by the backend provider
    """

    input_params: typing_extensions.Annotated[typing.Optional[JsonNode], FieldMetadata(alias="inputParams")] = None
    input_format: typing_extensions.Annotated[typing.Optional[JobInputFormat], FieldMetadata(alias="inputFormat")] = (
        None
    )
    tags: typing.Optional[typing.List[str]] = None
    status: typing.Optional[JobStatus] = None
    created_at: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="createdAt")] = None
    started_at: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="startedAt")] = None
    ended_at: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="endedAt")] = None
    shots: typing.Optional[int] = None
    sdk_provider: typing_extensions.Annotated[typing.Optional[JobSdkProvider], FieldMetadata(alias="sdkProvider")] = (
        None
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .azure_ionq_job_input_circuit_item import AzureIonqJobInputCircuitItem


class AzureIonqJobInput(UniversalBaseModel):
    """
    Azure IonQ input schema for job input (based on IonQ API Qis v0.4 qis-circuit reference)
    """

    circuit: typing.List[AzureIonqJobInputCircuitItem] = pydantic.Field()
    """
    List of gates to apply
    """

    gateset: typing.Literal["qis"] = pydantic.Field(default="qis")
    """
    Which gateset to apply
    """

    qubits: int = pydantic.Field()
    """
    Total number of qubits in the circuit
    """

    registers: typing.Optional[typing.Dict[str, typing.List[int]]] = pydantic.Field(default=None)
    """
    Registers to use in your circuit. Each register is a list of qubit indices (starting from zero).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

BackendProvider = typing.Union[
    typing.Literal["AZURE", "AWS", "IBM", "IBM_CLOUD", "DWAVE", "TSYSTEMS", "QRYD", "QUDORA", "QUANDELA", "IQM"],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

CostGranularity = typing.Union[
    typing.Literal["shot", "job", "1-Qubit Gate Shot", "2-Qubit Gate Shot", "minute", "second", "millisecond"],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .json_node import JsonNode


class CalibrationResponse(UniversalBaseModel):
    backend_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="backendId")] = pydantic.Field(
        default=None
    )
    """
    Unique identifier of the backend
    """

    calibrated_at: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="calibratedAt")] = None
    calibration: typing.Optional[JsonNode] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

DocumentationSdksItem = typing.Union[
    typing.Literal["PLANQK", "PENNYLANE", "QISKIT", "QISKIT_RUNTIME", "DWAVE", "QOQO", "PERCEVAL"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

AzureIonqJobInputCircuitItemType = typing.Union[
    typing.Literal[
        "x",
        "y",
        "z",
        "rx",
        "ry",
        "rz",
        "h",
        "s",
        "si",
        "v",
        "vi",
        "t",
        "ti",
        "not",
        "cnot",
        "swap",
        "xx",
        "yy",
        "zz",
        "pauliExp",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .azure_ionq_job_input_circuit_item_type import AzureIonqJobInputCircuitItemType


class AzureIonqJobInputCircuitItem(UniversalBaseModel):
    """
    A gate application to qubit in the circuit
    """

    type: typing.Optional[AzureIonqJobInputCircuitItemType] = pydantic.Field(default=None)
    """
    Gate name
    """

    targets: typing.List[int] = pydantic.Field()
    """
    Qubit indices the gate is applied to
    """

    controls: typing.Optional[typing.List[int]] = pydantic.Field(default=None)
    """
    Optional control-qubit indices
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

JobStatus = typing.Union[
    typing.Literal["UNKNOWN", "PENDING", "ABORTED", "RUNNING", "COMPLETED", "FAILED", "CANCELLING", "CANCELLED"],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class AwsQasm3JobInputParams(UniversalBaseModel):
    """
    Optional job parameters for AWS Braket QASM backends
    """

    disable_qubit_rewiring: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If true, automatic qubit rewiring is disabled, i.e. Braket is not allowed to re-map logical qubits to physical qubits on the backend device to optimize circuit execution. If not specified, the default is false.
    """

    qubit_count: typing.Optional[float] = pydantic.Field(default=None)
    """
    Number of qubits used in the circuit. If not specified, it is tried to infer it from the circuit.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



import json
import os
import platform
from abc import ABC, abstractmethod
from json import JSONDecodeError

_PERSONAL_ACCESS_TOKEN_NAME = "PLANQK_PERSONAL_ACCESS_TOKEN"
_CONFIG_FILE_PATH = "PLANQK_CONFIG_FILE_PATH"


class CredentialUnavailableError(Exception):
    """
    Exception raised when credentials are unavailable for authentication.
    """

    def __init__(self, message=None):
        if message is None:
            message = f"Credentials not found. Please set your personal access token as parameter, in the environment, or use 'planqk login' to authenticate."
        super().__init__(message)


class CredentialProvider(ABC):

    @abstractmethod
    def get_access_token(self) -> str:
        pass


class EnvironmentCredential(CredentialProvider):

    def get_access_token(self) -> str:
        access_token = os.environ.get(_PERSONAL_ACCESS_TOKEN_NAME)

        if not access_token:
            raise CredentialUnavailableError(f"Environment variable {_PERSONAL_ACCESS_TOKEN_NAME} is not set")

        return access_token


def get_config_file_path():
    config_file_path = os.environ.get(_CONFIG_FILE_PATH, None)
    if not config_file_path:
        if platform.system() == "Windows":
            config_dir = os.path.join(os.getenv("LOCALAPPDATA"), "planqk")
        else:
            config_dir = os.path.join(os.path.expanduser("~"), ".config", "planqk")
        config_file_path = os.path.join(config_dir, "config.json")
    return config_file_path


class ConfigFileCredential(CredentialProvider):
    def __init__(self):
        self.config_file = get_config_file_path()

    def get_access_token(self) -> str:
        if not self.config_file:
            raise CredentialUnavailableError("Config file location not set")
        if not os.path.isfile(self.config_file):
            raise CredentialUnavailableError(f"Config file at {self.config_file} does not exist")
        try:
            access_token = ConfigFileCredential.parse_file(self.config_file)
        except JSONDecodeError:
            raise CredentialUnavailableError("Failed to parse config file: Invalid JSON")
        except KeyError as e:
            raise CredentialUnavailableError(f"Failed to parse config file: Missing expected value - {str(e)}")
        except Exception as e:
            raise CredentialUnavailableError(f"Failed to parse config file: {str(e)}")
        return access_token

    @staticmethod
    def parse_file(path) -> str:
        with open(path, "r") as file:
            data = json.load(file)
            return data["auth"]["value"]


class StaticCredential(CredentialProvider):
    def __init__(self, access_token=None):
        self.access_token = access_token

    def get_access_token(self) -> str:
        if not self.access_token:
            raise CredentialUnavailableError(f"Access token not set")
        return self.access_token


class DefaultCredentialsProvider(CredentialProvider):
    def __init__(self, access_token=None):
        self.credentials = [
            StaticCredential(access_token),
            EnvironmentCredential(),
            ConfigFileCredential(),
        ]

    def get_access_token(self) -> str:
        for credential in self.credentials:
            try:
                return credential.get_access_token()
            except CredentialUnavailableError:
                # ignore and try the next credential provider
                pass

        raise CredentialUnavailableError()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.datetime_utils import serialize_datetime
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.forbidden_error import ForbiddenError
from ..errors.internal_server_error import InternalServerError
from ..errors.not_found_error import NotFoundError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.backend import Backend
from ..types.backend_state_info import BackendStateInfo
from ..types.calibration_response import CalibrationResponse


class RawBackendsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_backends(
        self,
        *,
        provider: typing.Optional[str] = None,
        only_planqk_sdk: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.List[Backend]]:
        """
        Get the list of backends based on the provided filters.

        Parameters
        ----------
        provider : typing.Optional[str]

        only_planqk_sdk : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[Backend]]
            The list of backends based on the provided filters.
        """
        _response = self._client_wrapper.httpx_client.request(
            "backends",
            method="GET",
            params={
                "provider": provider,
                "onlyPlanqkSdk": only_planqk_sdk,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[Backend],
                    parse_obj_as(
                        type_=typing.List[Backend],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_backend(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[Backend]:
        """
        Get the details of a specific backend.

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Backend]
            The backend with the specified ID.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"backends/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Backend,
                    parse_obj_as(
                        type_=Backend,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_backend_status(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[BackendStateInfo]:
        """
        Get the status of a specific backend.

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BackendStateInfo]
            The status of the backend.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"backends/{jsonable_encoder(id)}/status",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BackendStateInfo,
                    parse_obj_as(
                        type_=BackendStateInfo,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_backend_config(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.Dict[str, typing.Any]]:
        """
        Get the configuration of a specific backend.

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Dict[str, typing.Any]]
            The configuration of the backend.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"backends/{jsonable_encoder(id)}/config",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Dict[str, typing.Any],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_backend_calibration(
        self,
        id: str,
        *,
        effective_at: typing.Optional[dt.datetime] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CalibrationResponse]:
        """
        Get the calibration of a specific backend.

        Parameters
        ----------
        id : str

        effective_at : typing.Optional[dt.datetime]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CalibrationResponse]
            The backend's calibration.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"backends/{jsonable_encoder(id)}/calibration",
            method="GET",
            params={
                "effectiveAt": serialize_datetime(effective_at) if effective_at is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CalibrationResponse,
                    parse_obj_as(
                        type_=CalibrationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawBackendsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_backends(
        self,
        *,
        provider: typing.Optional[str] = None,
        only_planqk_sdk: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.List[Backend]]:
        """
        Get the list of backends based on the provided filters.

        Parameters
        ----------
        provider : typing.Optional[str]

        only_planqk_sdk : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[Backend]]
            The list of backends based on the provided filters.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "backends",
            method="GET",
            params={
                "provider": provider,
                "onlyPlanqkSdk": only_planqk_sdk,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[Backend],
                    parse_obj_as(
                        type_=typing.List[Backend],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_backend(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Backend]:
        """
        Get the details of a specific backend.

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Backend]
            The backend with the specified ID.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"backends/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Backend,
                    parse_obj_as(
                        type_=Backend,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_backend_status(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[BackendStateInfo]:
        """
        Get the status of a specific backend.

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BackendStateInfo]
            The status of the backend.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"backends/{jsonable_encoder(id)}/status",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BackendStateInfo,
                    parse_obj_as(
                        type_=BackendStateInfo,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_backend_config(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.Dict[str, typing.Any]]:
        """
        Get the configuration of a specific backend.

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Dict[str, typing.Any]]
            The configuration of the backend.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"backends/{jsonable_encoder(id)}/config",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Dict[str, typing.Any],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_backend_calibration(
        self,
        id: str,
        *,
        effective_at: typing.Optional[dt.datetime] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CalibrationResponse]:
        """
        Get the calibration of a specific backend.

        Parameters
        ----------
        id : str

        effective_at : typing.Optional[dt.datetime]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CalibrationResponse]
            The backend's calibration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"backends/{jsonable_encoder(id)}/calibration",
            method="GET",
            params={
                "effectiveAt": serialize_datetime(effective_at) if effective_at is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CalibrationResponse,
                    parse_obj_as(
                        type_=CalibrationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



import os
from typing import Optional

from planqk.quantum.client.credentials import DefaultCredentialsProvider
from planqk.quantum.client.sdk import GeneratedPlanqkQuantumClient

_PLANQK_QUANTUM_API_BASE_URL_NAME = "PLANQK_QUANTUM_API_BASE_URL"


class PlanqkQuantumClient:
    def __init__(self, access_token: Optional[str] = None, organization_id: Optional[str] = None):
        base_url = os.environ.get(_PLANQK_QUANTUM_API_BASE_URL_NAME, "https://platform.planqk.de/quantum")
        credentials_provider = DefaultCredentialsProvider(access_token)

        self._client = GeneratedPlanqkQuantumClient(base_url=base_url, api_key=credentials_provider.get_access_token(),
                                                    organizationid=organization_id)
        self.backends = self._client.backends
        self.jobs = self._client.jobs



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .configuration_output_format import ConfigurationOutputFormat
from .configuration_supported_input_formats_item import ConfigurationSupportedInputFormatsItem
from .connectivity import Connectivity
from .gate import Gate
from .photonic_properties import PhotonicProperties
from .qubit import Qubit
from .shots_range import ShotsRange


class Configuration(UniversalBaseModel):
    gates: typing.Optional[typing.List[Gate]] = None
    operations: typing.Optional[typing.List[str]] = None
    qubit_count: typing.Optional[int] = None
    connectivity: typing.Optional[Connectivity] = None
    shots_range: typing.Optional[ShotsRange] = None
    supported_input_formats: typing.Optional[typing.List[ConfigurationSupportedInputFormatsItem]] = None
    output_format: typing.Optional[ConfigurationOutputFormat] = None
    memory_result_supported: typing.Optional[bool] = None
    photonic_properties: typing.Optional[PhotonicProperties] = None
    qubits: typing.Optional[typing.List[Qubit]] = None
    instructions: typing.Optional[typing.List[str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
